#include "4DPluginAPI.h"
#include "Imageroutinen.h"
#include "altura_resource.h"


#if VERSIONWIN
	#include "atlbase.h"  // for debugging
	static 	ULONG_PTR   gdiplusToken;
	#define strlcat(a, b, c) strcat_s(a, c, b)
	#define strlcpy(a, b, c) strcpy_s(a, c, b)
#else
#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif
#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#endif

void NewOffscreen(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle; 

	AreaHandle = (ImageAreaStruct *) malloc(sizeof(ImageAreaStruct));
	InitAreaObjects(AreaHandle);
	AreaHandle->Offscreen = true;
	PA_ReturnLong( params, (long) AreaHandle );
}


void DeleteOffscreen(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle; 

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if (AreaHandle->Offscreen)
		{	ClearImage(AreaHandle);
			free(AreaHandle);
		}
}

void PreInitArea(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle; 
	PA_PluginProperties currentPluginProps;	 
	
	//  bug?? ACI0026601   
	// PA_GetPluginProperties seems not to work in InitArea. So do it here and copy results.

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if (AreaHandle == 0L)
	{
		PA_GetPluginProperties(params, &currentPluginProps);
		AreaHandle = (ImageAreaStruct *) malloc(sizeof(ImageAreaStruct));
		if( AreaHandle != NULL )
		{	InitAreaObjects(AreaHandle);		
			AreaHandle->FormPage = currentPluginProps.fPage;
			AreaHandle->AllowDrag = (currentPluginProps.fDraggable != 0);
			AreaHandle->TableNumber = currentPluginProps.fTable;
			AreaHandle->Offscreen = false;
			PA_SetAreaReference( params, AreaHandle );
			AreaHandle->AreaProcess = PA_GetCurrentProcessNumber();
#if VERSIONWIN
			AreaHandle->FormWindow = (HWND) currentPluginProps.fWinHWND;
			AreaHandle->fWinHDC = (HDC) currentPluginProps.fWinHDC;	
#else
			AreaHandle->FormWindow = (CWindowPtr) currentPluginProps.fMacWindow;
			AreaHandle->fMacPort = (CGrafPtr) currentPluginProps.fMacPort;	
#endif	
		}
	}
}

void InitArea(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle; 
#if VERSIONWIN
	HANDLE myWindowProp;
#endif	

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );

	if(!AreaHandle)  // should be set from PreInitArea, but may be called if ACI0026601 changes
	{
		AreaHandle = (ImageAreaStruct *) malloc(sizeof(ImageAreaStruct));
		if( AreaHandle != NULL )
		{	InitAreaObjects(AreaHandle);	
			AreaHandle->FormPage = 1;  // unknown - error
			PA_SetAreaReference( params, AreaHandle );
		}
	}

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) )
	{
		PA_Rect myrect = PA_GetAreaRect (params);		
		AreaHandle->oldSize = myrect;

		#if VERSIONWIN
			// native Windows Scrollbars only working with 2004 or newer, see discussion in PlugIn Talk List, RE : 4D 2004 scrollbars, 11/10/2004
			// because it seems to work fine with 2003 it is enabled for 2003.
			// if you have problems, use the language to disable the scrollbars.
			if ( ( PA_Get4DVersion() & 0x0000FFFF ) >= kMINIMAL_4DVERSION ) 
			{	AreaHandle->HorScroll = CreateWindowEx( 
					0L,                          // no extended styles 
					"SCROLLBAR",                 // scroll bar control class 
					(LPSTR) NULL,                // text for window title bar 
					WS_CHILD | SBS_HORZ,         // scroll bar styles 
					myrect.fLeft,                           // horizontal position 
					myrect.fTop+(myrect.fBottom-myrect.fTop-AreaScrollbarSize),                           // vertical position 
					myrect.fRight-myrect.fLeft-AreaScrollbarSize,                         // width of the scroll bar 
					AreaScrollbarSize, //CW_USEDEFAULT,               // default height 
					AreaHandle->FormWindow,                   // handle to main window 
					(HMENU) NULL,           // no menu for a scroll bar 
					(HINSTANCE) PA_Get4DHInstance(),                  // instance owning this window 
					(LPVOID) NULL           // pointer not needed 
					);

				AreaHandle->VertScroll = CreateWindowEx( 
					0L,                          // no extended styles 
					"SCROLLBAR",                 // scroll bar control class 
					(LPSTR) NULL,                // text for window title bar 
					WS_CHILD | SBS_VERT,         // scroll bar styles 
					myrect.fLeft+(myrect.fRight-myrect.fLeft-AreaScrollbarSize),         // horizontal position 
					myrect.fTop,                           // vertical position 
					AreaScrollbarSize, //CW_USEDEFAULT,               // default width 
					myrect.fBottom-myrect.fTop-AreaScrollbarSize,                         // height of the scroll bar 
					AreaHandle->FormWindow,                   // handle to main window 
					(HMENU) NULL,           // no menu for a scroll bar 
					(HINSTANCE) PA_Get4DHInstance(),                  // instance owning this window 
					(LPVOID) NULL           // pointer not needed 
					);

				if(AreaHandle->HorScroll)
				{
					SetProp(AreaHandle->HorScroll, "ScrollAreaHandle", AreaHandle); 
					if (AreaHandle->wpOrigWinProc == 0L)
					{
						myWindowProp = GetProp(AreaHandle->FormWindow, "GDIScrollHandler");
						if (myWindowProp == 0)
						{	AreaHandle->wpOrigWinProc = (WNDPROC) SetWindowLong(GetParent(AreaHandle->HorScroll), 
								GWL_WNDPROC, (LONG) ScrollWndProc); 
							SetProp(AreaHandle->FormWindow, "GDIScrollHandler", AreaHandle->wpOrigWinProc);
						}
					}

				}
				if(AreaHandle->VertScroll)
				{
					SetProp(AreaHandle->VertScroll, "ScrollAreaHandle", AreaHandle); 
				}
			}
		#else
		// MacOS Scrollbars   
			Rect scrollposition;
			SetRect(&scrollposition, myrect.fLeft, myrect.fBottom-AreaScrollbarSize, myrect.fRight, myrect.fBottom);
			AreaHandle->myControlActionUPP = NewControlActionUPP(MyScrollbarActionProc); 
			OSStatus err = CreateScrollBarControl(AreaHandle->FormWindow, &scrollposition, 0, 0, 10, 1, true, AreaHandle->myControlActionUPP, &AreaHandle->HorScroll);
			if (!err)
				SetControlReference(AreaHandle->HorScroll, (SInt32) AreaHandle);
			SetRect(&scrollposition, myrect.fRight-AreaScrollbarSize, myrect.fTop, myrect.fRight, myrect.fBottom);
			err = CreateScrollBarControl(AreaHandle->FormWindow, &scrollposition, 0, 0, 10, 1, true, AreaHandle->myControlActionUPP, &AreaHandle->VertScroll);
			if (!err)
				SetControlReference(AreaHandle->VertScroll, (SInt32) AreaHandle);
		#endif

		PA_SetAreaReference( params, AreaHandle );	
		LoadAssignedField(params);
	}
}

void DeinitArea(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle;
#if VERSIONWIN	
	long myWindowProp;
	HWND hwnd;
#endif	

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) )
	{
		ClearImage(AreaHandle);
#if VERSIONWIN		
			hwnd = GetParent(AreaHandle->HorScroll);
			myWindowProp = (long) GetProp(hwnd, "GDIScrollHandler");  // only remove if it is still set
			// this handles the case if there are several areas in the same form
			if (myWindowProp != 0)
			{	RemoveProp(hwnd, "GDIScrollHandler");
				SetWindowLong(hwnd, GWL_WNDPROC, (LONG) AreaHandle->wpOrigWinProc); 
			}
			if (AreaHandle->HorScroll != 0L)
			{	RemoveProp(AreaHandle->HorScroll, "ScrollAreaHandle"); 
				DestroyWindow(AreaHandle->HorScroll );  }
			if (AreaHandle->VertScroll != 0L)
			{	RemoveProp(AreaHandle->VertScroll, "ScrollAreaHandle"); 
				DestroyWindow(AreaHandle->VertScroll );  }
#else  // MacOS
			if (AreaHandle->myControlActionUPP)
				DisposeControlActionUPP(AreaHandle->myControlActionUPP);
			if (AreaHandle->HorScroll)
				DisposeControl(AreaHandle->HorScroll);
			if (AreaHandle->VertScroll)
				DisposeControl(AreaHandle->VertScroll);
#endif

		PA_SetAreaReference( params, 0 );
		free(AreaHandle);
	}
}



void InitAreaObjects(ImageAreaStruct *AreaHandle)
{
	if (AreaHandle != NULL) 
	{
	AreaHandle->TAG = kAreaTag;  // just a tag to recognize/verify our struct
	AreaHandle->ZoomX = AreaHandle->ZoomY = 0;
#if VERSIONWIN
	AreaHandle->image = NULL;
	AreaHandle->wpOrigWinProc = 0L;
	AreaHandle->HorScroll = AreaHandle->VertScroll = 0L;
	AreaHandle->pStream = NULL;
#else
	// Mac scroll
	AreaHandle->fMacPort = NULL;	
	AreaHandle->image = NULL;
	AreaHandle->HorScroll = AreaHandle->VertScroll = 0L;
	AreaHandle->myControlActionUPP = NULL;
#endif
	AreaHandle->FormWindow = NULL;
	AreaHandle->ScrollX = AreaHandle->ScrollY = 0;
	AreaHandle->FormPage = -1;
	AreaHandle->HScrollActive = AreaHandle->VScrollActive = false;
	AreaHandle->HScrollShow = true;
	AreaHandle->VScrollShow = true;
	AreaHandle->RedrawRequired = false;
	AreaHandle->Dirty = false;
	AreaHandle->AutoDrop = true;
	AreaHandle->AllowCopy = true;
	AreaHandle->AllowPaste = true;
	}
}


void AreaChangeFormPage(PA_PluginParameters params)  // called if switched to another page
{
	ImageAreaStruct *AreaHandle;
	PA_Event* event;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{
		event = ( (PA_Event**) params->fParameters )[ 0 ];
		if (event->fWhen == AreaHandle->FormPage)
		{	// display our scrollbars
			#if VERSIONWIN		
				if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
					ShowWindow(AreaHandle->HorScroll, SW_SHOWNORMAL); 
				if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
					ShowWindow(AreaHandle->VertScroll, SW_SHOWNORMAL); 
			#else  // MacOS
				if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
					ShowControl(AreaHandle->HorScroll); 
				if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
					ShowControl(AreaHandle->VertScroll); 
			#endif
		}
		else
		{	// hide our scrollbars
			#if VERSIONWIN		
				if (AreaHandle->HorScroll)
					ShowWindow(AreaHandle->HorScroll, SW_HIDE); 
				if (AreaHandle->VertScroll)
					ShowWindow(AreaHandle->VertScroll, SW_HIDE); 
			#else  // MacOS
				if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
					HideControl(AreaHandle->HorScroll); 
				if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
					HideControl(AreaHandle->VertScroll); 
#endif
		}
	}
}

void CheckRedrawRequired(PA_PluginParameters params)  // called from EndExecutionClycle
{
	ImageAreaStruct *AreaHandle;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if (AreaHandle->RedrawRequired)
		{	AreaHandle->RedrawRequired = false;
			PA_RequestRedraw (params);
		}
}

void AreaMouseClick(PA_PluginParameters params)
{
#if VERSIONWIN		
	CheckDragOut(params);
#else
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if (AreaHandle->HorScroll || AreaHandle->VertScroll) 
		{
		short x, y;
		PA_GetClick( params, &x, &y );
		EventModifiers Modifiers = PA_GetModifiers(params);
		Point MouseClick;
		MouseClick.h = x;
		MouseClick.v = y;
		ControlPartCode outPart;
		ControlRef theScrollbar = FindControlUnderMouse(MouseClick, AreaHandle->FormWindow, &outPart);
		if (theScrollbar)
			HandleControlClick(theScrollbar, MouseClick, Modifiers, AreaHandle->myControlActionUPP); //MyScrollbarActionProc);
		else
			CheckDragOut(params);
		}
	
#endif
}

void CheckDragOut(PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{
		if (AreaHandle->AllowDrag)
		{
			PA_Rect      areaRect;
			short      x, y;
   
			PA_GetClick(params, &x, &y);
			if(PA_GetLastError() == eER_NoErr)
			 {
// We must convert those local coordinates to global
#if VERSIONMAC
				  Point   p;
				  p.h = x;
				  p.v = y;
				  LocalToGlobal(&p);
				  x = p.h;
				  y = p.v;
#else
				  POINT   p;
			      
				  p.x = x;
				  p.y = y;
				  ClientToScreen( AreaHandle->FormWindow, &p);
				  x = (short) p.x;
				  y = (short) p.y;
#endif
// prepare the returned string
      //PA_MoveBlock(privateData->toDraw, gToDraw, 256);
// Start the drag
			areaRect = PA_GetAreaRect(params);
			PA_DragAndDrop(x, y, 1, areaRect);
			}
		}
	}
}

void AreaUpdate(PA_PluginParameters params)
{
#if VERSIONWIN		
	PA_Rect myrect;
	ImageAreaStruct *AreaHandle;
	HDC hdc;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	myrect = PA_GetAreaRect (params);

		hdc = (HDC) PA_GetUpdateHDC();
		if( hdc != AreaHandle->fWinHDC) 
		{	// this is really strange. there seems to be additional redraw events even for invisible areas.
			// if filtered like here we remove "flicker" redraws and problems during resize for invisble areas.
			DrawImage(AreaHandle, hdc, myrect);
		}
		if (AreaHandle->HorScroll || AreaHandle->VertScroll)  
		{   

			if ( (AreaHandle->oldSize.fBottom != myrect.fBottom) || (AreaHandle->oldSize.fLeft != myrect.fLeft) ||
				(AreaHandle->oldSize.fTop != myrect.fTop) || (AreaHandle->oldSize.fRight != myrect.fRight))
			{  // new size
				long VOffset, HOffset;
				if (AreaHandle->VScrollActive)
					VOffset=AreaScrollbarSize;
				else
					VOffset=0;
				if (AreaHandle->HScrollActive)
					HOffset=AreaScrollbarSize;
				else
					HOffset=0;

				MoveWindow(AreaHandle->HorScroll, 
						myrect.fLeft,                           // horizontal position 
						myrect.fTop+(myrect.fBottom-myrect.fTop-AreaScrollbarSize),                           // vertical position 
						myrect.fRight-myrect.fLeft-VOffset,                         // width of the scroll bar 
						AreaScrollbarSize,
						FALSE);
				MoveWindow(AreaHandle->VertScroll, 
						myrect.fLeft+(myrect.fRight-myrect.fLeft-AreaScrollbarSize),              // horizontal position 
						myrect.fTop,                           // vertical position 
						AreaScrollbarSize,                         // width of the scroll bar 
						myrect.fBottom-myrect.fTop-HOffset,
						FALSE);
			}
			AreaHandle->oldSize = myrect;  // update, needed for scrollbar

			if (AreaHandle->HScrollActive || AreaHandle->VScrollActive)
			{
				// windows bug: some controls do not properly restore dc settings.
				// such as the brush and the text alignment.
				HGDIOBJ oldBrush = ::SelectObject( hdc, ::GetStockObject(WHITE_BRUSH));
				UINT oldalign = ::SetTextAlign(hdc,0);
				// move dc origin to the control origin
				POINT pt;
				pt.x = pt.y = 0;

				if (AreaHandle->HScrollActive)
				{	::MapWindowPoints( GetParent(AreaHandle->HorScroll), AreaHandle->HorScroll, &pt, 1);
					::OffsetWindowOrgEx( hdc, pt.x, pt.y, &pt);
					// ask the control to draw itself in specified dc
					::SendMessage( AreaHandle->HorScroll, WM_PRINTCLIENT, (WPARAM) hdc, PRF_CLIENT | PRF_OWNED | PRF_CHILDREN);
					// restore dc origin
					::SetWindowOrgEx( hdc, pt.x, pt.y, NULL);
				}

				if (AreaHandle->VScrollActive)
				{	::MapWindowPoints( GetParent(AreaHandle->VertScroll), AreaHandle->VertScroll, &pt, 1);
					::OffsetWindowOrgEx( hdc, pt.x, pt.y, &pt);
					// ask the control to draw itself in specified dc
					::SendMessage( AreaHandle->VertScroll, WM_PRINTCLIENT, (WPARAM) hdc, PRF_CLIENT | PRF_OWNED | PRF_CHILDREN);
					// restore dc origin
					::SetWindowOrgEx( hdc, pt.x, pt.y, NULL);
				}

				// restore dc settings that may have been modified by WM_PRINTCLIENT
				oldalign = ::SetTextAlign( hdc, oldalign);
				oldBrush = ::SelectObject( hdc, oldBrush);
			}
		}

	}
#else  
// MacOS AreaUpdate(PA_PluginParameters params)
	PA_Rect myrect;
	ImageAreaStruct *AreaHandle;
	CGrafPtr port;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	myrect = PA_GetAreaRect (params);
		GetPort(&port);
		//if (AreaHandle->fMacPort != port)
			DrawImage(AreaHandle, myrect);
		if (AreaHandle->HorScroll || AreaHandle->VertScroll)  
		{
				long VOffset, HOffset;
				if (AreaHandle->VScrollActive)
					VOffset=AreaScrollbarSize;
				else
					VOffset=0;
				if (AreaHandle->HScrollActive)
					HOffset=AreaScrollbarSize;
				else
					HOffset=0;

				if ( (AreaHandle->oldSize.fBottom != myrect.fBottom) || (AreaHandle->oldSize.fLeft != myrect.fLeft) ||
				 (AreaHandle->oldSize.fTop != myrect.fTop) || (AreaHandle->oldSize.fRight != myrect.fRight))
				{  // new size
					MoveControl(AreaHandle->HorScroll,  myrect.fLeft, myrect.fBottom-AreaScrollbarSize);
					SizeControl(AreaHandle->HorScroll, 	myrect.fRight-myrect.fLeft-VOffset, AreaScrollbarSize);
					
					MoveControl(AreaHandle->VertScroll, myrect.fRight-AreaScrollbarSize, myrect.fTop);
					SizeControl(AreaHandle->VertScroll, AreaScrollbarSize, myrect.fBottom-myrect.fTop-HOffset);			
				}
				AreaHandle->oldSize = myrect;  // update, needed for scrollbar
		
			if (AreaHandle->HScrollActive)
				DrawControlInCurrentPort(AreaHandle->HorScroll);
			if (AreaHandle->VScrollActive)
				DrawControlInCurrentPort(AreaHandle->VertScroll);
		}
	}
#endif
}

void AreaStructure(PA_PluginParameters params)
{
	char AreaName[80];
	PA_GetAreaName(params, AreaName);
	PA_Rect myrect = PA_GetAreaRect (params);

#if VERSIONWIN	
	HDC hdc;
	wchar_t wideAreaName[80];
	size_t Anzahlchar;

	hdc = (HDC) PA_GetUpdateHDC();

	Graphics graphics(hdc);
	Pen blackPen(Color(255, 0, 0, 0), 3);
	graphics.DrawRectangle(&blackPen, myrect.fLeft, myrect.fTop, (myrect.fRight-myrect.fLeft), (myrect.fBottom-myrect.fTop));
	SolidBrush  brush(Color(255, 0, 0, 255));
    FontFamily  fontFamily(L"Arial");
    Font        font(&fontFamily, 14, FontStyleRegular, UnitPixel);
    PointF      pointF(myrect.fLeft + 10.0f, myrect.fTop + 10.0f);
	errno_t err = mbstowcs_s(&Anzahlchar,  wideAreaName, 80, AreaName, _TRUNCATE);
	TCHAR Message[255];
	wchar_t MessageW[255];
	HINSTANCE  myinstance = GetModuleHandle("PictContainer.4DX");
	long Fieldtyp = CheckAssignedField(params, true);
	switch (Fieldtyp)
		{
		case (eFK_PictureField):
			Anzahlchar = LoadString(myinstance, 102, Message, sizeof(Message)/sizeof(TCHAR)); break;
		case (eFK_BlobField):
			Anzahlchar = LoadString(myinstance, 103, Message, sizeof(Message)/sizeof(TCHAR)); break;
		case (eFK_TextField):
			Anzahlchar = LoadString(myinstance, 104, Message, sizeof(Message)/sizeof(TCHAR)); break;
		default:
			Anzahlchar = LoadString(myinstance, 105, Message, sizeof(Message)/sizeof(TCHAR)); break;
		}
	err = mbstowcs_s(&Anzahlchar,  MessageW, 80, Message, _TRUNCATE);
	graphics.DrawString(MessageW, -1, &font, pointF, &brush);
	pointF.Y += 25;
	graphics.DrawString(wideAreaName, -1, &font, pointF, &brush);
	pointF.Y += 25;
	Anzahlchar = LoadString(myinstance, 101, Message, sizeof(Message)/sizeof(TCHAR));
	err = mbstowcs_s(&Anzahlchar,  MessageW, 80, Message, _TRUNCATE);
	graphics.DrawString(MessageW, -1, &font, pointF, &brush);

#else  
// MacOS AreaStructure(PA_PluginParameters params)
	unsigned char szTitle[255];
	FrameRect((const Rect*) &myrect);
	MoveTo(myrect.fLeft+5 , myrect.fTop + 20);
	unsigned char MacAreaName[255];
	PA_ConvertStrings ((char *) AreaName, eSK_CString, eCS_Macintosh, (char *) MacAreaName, eSK_PString, eCS_Macintosh);
	short Fieldtyp = CheckAssignedField(params, true);
	switch (Fieldtyp)
		{
		case (eFK_PictureField):
			GetIndString(szTitle, 15001, 2); break;
		case (eFK_BlobField):
			GetIndString(szTitle, 15001, 3); break;
		case (eFK_TextField):
			GetIndString(szTitle, 15001, 4); break;
		default:
			GetIndString(szTitle, 15001, 5); break;			
		}
	DrawString((const unsigned char *) szTitle);	
	DrawString((const unsigned char *) MacAreaName);
//	const unsigned char szTitle[] = "\pPicture Container 1.0 -  4D Germany - written by Thomas Maul, 4D Germany (May 2006)";
	GetIndString(szTitle, 15001, 1);
	MoveTo(myrect.fLeft+5 , myrect.fTop + 40);
	DrawString((const unsigned char *) szTitle);
#endif
}

void InitPlugin()
{
	PA_UseCStrings();
	PA_ReturnNullTerminatedText(1);

#if VERSIONWIN	
	GdiplusStartupInput gdiplusStartupInput;
    // Initialize GDI+.
   GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
#endif
}

void DeinitPlugin(void)
{	
#if VERSIONWIN	
	GdiplusShutdown(gdiplusToken);
#endif
}

#if VERSIONWIN
// nothing needs to be done for Windows
#else
void StorePictureInformations(ImageAreaStruct *AreaHandle, GraphicsImportComponent gi)
{
	ImageDescriptionHandle desc = NULL;
	
	long err = GraphicsImportGetImageDescription( gi,	// importer instance
											 &desc );	// ptr to ImageDescriptionHandle
	if( (!err) && desc && *desc ) {
		// print basic statistics from the image description.
		AreaHandle->PictWidth = (*desc)->width;
		AreaHandle->PictHeight = (*desc)->height ;
		AreaHandle->PictDepth = (*desc)->depth ;
		short length = sizeof(AreaHandle->Format);
		if (length > 31)
			{ length = 31; 	(*desc)->name[0] = 31; }				
		BlockMoveData( (*desc)->name,AreaHandle->Format, length );
		CopyPascalStringToC(AreaHandle->Format, (char *)AreaHandle->Format);
		AreaHandle->PictResX = FixedToFloat((*desc)->hRes);
		AreaHandle->PictResY = FixedToFloat((*desc)->vRes) ;
	}
	else
		{
		AreaHandle->PictWidth = 0;
		AreaHandle->PictHeight = 0;
		AreaHandle->PictDepth = 0;
		AreaHandle->Format[0] = 0;
		AreaHandle->PictResX = 0;
		AreaHandle->PictResY = 0;		
		}
	
}
#endif




void AllowDrop(PA_PluginParameters params)
{
 	ImageAreaStruct *AreaHandle;
	PA_DragAndDropInfo   info;
	PA_DragKind         kind;
	short            allow = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{
    
		if (AreaHandle->AllowNextDrop)
			allow = 1;
		else
			if (AreaHandle->AutoDrop)
			{
			info = PA_GetDragAndDropInfo ( params );
		   if(PA_GetLastError() == eER_NoErr)
		   {
		// Get the kind of object : variable or field
			  kind = PA_GetDragAndDropKind( info );
		// Check if it is a picture
			  switch(kind)
			  {
				 case eDK_DragVariable :
					PA_Variable         var;
					PA_VariableKind      varKind;
					var = PA_GetDragAndDropVariable( info, 0 );
					if (AreaHandle->AreaProcess == info.fFromProcess)
					{
						varKind = PA_GetVariableKind(var);
						if( (varKind == eVK_Picture ) || (varKind == eVK_ArrayPicture))
						   allow = 1;
					}
					break;
		            
				 case eDK_DragField :
					short         table, field;
					PA_FieldKind   fieldKind;

					PA_GetDragAndDropTableField( info, &table, &field );
				 // If table is 0, it means that a field of the table that
				 // owns the form is being dragged. We must use the table number
				 // stored at kInitArea
					if (AreaHandle->AreaProcess == info.fFromProcess)
					{
						if(table == 0)
							table = AreaHandle->TableNumber;
						if(table)
						{
						   PA_GetFieldProperties(table, field, &fieldKind, 0L, 0L, 0L);
						   if( fieldKind == eFK_PictureField )
							  allow =  1;
						}
					}
					break;
					
					default: 
					break;  // nothing
			  }
		   }
	   }
	}
	AreaHandle->AllowNextDrop = 0;  // reset flag, maybe it was used
	PA_AllowDrop( params, (char) allow );
}

void AreaDrop(PA_PluginParameters params)
{
 	ImageAreaStruct *AreaHandle;
	PA_DragAndDropInfo   info;
	PA_DragKind         kind;
	PA_PictureInfo Picture_info;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{
    
	   info = PA_GetDragAndDropInfo ( params );
	   if(PA_GetLastError() == eER_NoErr)
	   {
	// Get the kind of object : variable or field
		  kind = PA_GetDragAndDropKind( info );
	// Check if it is a picture
		  switch(kind)
		  {
			 case eDK_DragVariable :
				PA_Variable         var;
				PA_VariableKind      varKind;

				long index;
				var = PA_GetDragAndDropVariable( info, &index );
				if (AreaHandle->AreaProcess == info.fFromProcess)  // we can handle only the same process!
				{
					varKind = PA_GetVariableKind(var);
					if( varKind == eVK_Picture )
					{	AreaHandle->Dirty = true;
						Load4DPictureInHandle(AreaHandle, PA_GetPictureHandleVariable(var, &Picture_info)); }
					else
					if  (varKind == eVK_ArrayPicture)
					{	AreaHandle->Dirty = true;
						Load4DPictureInHandle(AreaHandle, PA_GetPictureHandleInArray(var, info.fFromArrayIndice, &Picture_info));   }
				}	
				break;
	            
			 case eDK_DragField :
				short         table, field;
				PA_FieldKind   fieldKind;

				PA_GetDragAndDropTableField( info, &table, &field );
				if (AreaHandle->AreaProcess == info.fFromProcess)  // we can handle only the same process!
				{
					if(table == 0)
						table = AreaHandle->TableNumber;
					if(table)
					{
					   PA_GetFieldProperties(table, field, &fieldKind, 0L, 0L, 0L);
					   if( fieldKind == eFK_PictureField )
					   {	AreaHandle->Dirty = true;
							Load4DPictureInHandle(AreaHandle, PA_GetPictureHandleField (table, field, &Picture_info)); }
					}
				}
				break;
				
				default: 
				break;  // nothing		 
				 }
	   }
	}
}




// ------------------------------- Get/Set options  ------------------------------


long LoadPictureFile( ImageAreaStruct *AreaHandle, char * Pfad )
{
	long returnValue = 0;
	
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) & (strlen(Pfad) != 0))
		{	
		ClearImage(AreaHandle);
#if VERSIONWIN	
		long Pfad_len = strlen(Pfad);
		wchar_t *pfadname = (wchar_t *)malloc( (Pfad_len+10) * 2);
		size_t Anzahlchar;
		errno_t err = mbstowcs_s(&Anzahlchar, pfadname, (Pfad_len+10), Pfad, _TRUNCATE);
		
		AreaHandle->image = Image::FromFile(pfadname, true); 
		returnValue = AreaHandle->image->GetLastStatus();
		free(pfadname);
#else
		FSSpec myFSSpec;
		OSErr err = build_FSSpecFrompath ( Pfad, &myFSSpec);
		if (err == noErr)
			{
			GraphicsImportComponent gi;
			GetGraphicsImporterForFile(&myFSSpec, &gi);
			StorePictureInformations(AreaHandle, gi);
			err = GraphicsImportGetAsPicture(gi, &AreaHandle->image);
			CloseComponent( gi);
			}
		
		returnValue = err;
#endif
		AreaHandle->RedrawRequired = true;
		}
	return returnValue;
}



void LoadPictureFile( PA_PluginParameters params )
{
	long Pfad_len;
	char *Pfad;
	long returnValue = 0;
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		Pfad_len = PA_GetTextParameter( params, 2, 0L );	
		Pfad = (char *) malloc(Pfad_len+1);
		Pfad_len = PA_GetTextParameter( params, 2, Pfad );
		Pfad[Pfad_len] = 0;
		
		returnValue = LoadPictureFile(AreaHandle, Pfad);
		
		free(Pfad);
		}
	else
		returnValue = kError_unknownArea;
	
	PA_ReturnLong( params, returnValue );
}

long SavePictureFile( ImageAreaStruct *AreaHandle, char * Pfad, char * format, long compression )
{
	long returnValue = 0;
	
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) & (strlen(Pfad) != 0))
		{	
#if VERSIONWIN	
		CLSID pngClsid;
		if (GetEncoderClsid(format, pngClsid) <0)
			return -1; 

		long Pfad_len = strlen(Pfad);
		wchar_t *pfadname = (wchar_t *)malloc( (Pfad_len+10) * 2);
		size_t Anzahlchar;
		errno_t err = mbstowcs_s(&Anzahlchar, pfadname, (Pfad_len+10), Pfad, _TRUNCATE);
		
		if (!strcmp(format, "jpg"))
		{
			EncoderParameters* 	encoderParameters = (EncoderParameters*) malloc(sizeof(EncoderParameters));
			encoderParameters->Count = 1;
			encoderParameters->Parameter[0].Guid = EncoderQuality;
			encoderParameters->Parameter[0].Type = EncoderParameterValueTypeLong;
			encoderParameters->Parameter[0].NumberOfValues = 1;
			ULONG  quality = compression;
			encoderParameters->Parameter[0].Value = &quality;

			AreaHandle->image->Save(pfadname, &pngClsid, encoderParameters);
			free(encoderParameters);
		}
		else
			AreaHandle->image->Save(pfadname, &pngClsid, NULL); 

		returnValue = AreaHandle->image->GetLastStatus();
		free(pfadname);
#else
		FSSpec myFSSpec;
			
		OSErr err = build_FSSpecFrompathNewfile ( Pfad, &myFSSpec);
		if (err == noErr)
			{
			GraphicsExportComponent exporter;
			
			err = OpenADefaultComponent( GraphicsExporterComponentType,  GetEncoderType(format),   &exporter );  
			
			if (!err)
				err = GraphicsExportSetInputPicture (exporter, AreaHandle->image);
			if (!err)
				err = GraphicsExportSetOutputFile (exporter,  &myFSSpec);
			if ((!err) && (compression)	)
			   err = GraphicsExportSetCompressionQuality( exporter,  GetCompression( compression) );
			if (!err)
				err = GraphicsExportDoExport (exporter, NULL);
				
			 CloseComponent ( exporter );	
			}
 		
		returnValue = err;
#endif
		}
	return returnValue;
}

void SavePictureFile( PA_PluginParameters params )
{
	long Pfad_len;
	char *Pfad;
	char format[255];
	long returnValue = 0;
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		Pfad_len = PA_GetTextParameter( params, 2, 0L );	
		Pfad = (char *) malloc(Pfad_len+1);
		Pfad_len = PA_GetTextParameter( params, 2, Pfad );
		Pfad[Pfad_len] = 0;
		PA_GetStringParameter(params, 3, format);
		long compression =PA_GetLongParameter(params, 4);

		returnValue = SavePictureFile(AreaHandle, Pfad, format, compression);
		
		free(Pfad);
		}
	else
		returnValue = kError_unknownArea;
	
	PA_ReturnLong( params, returnValue );
}

long SavePictureBlob(ImageAreaStruct *AreaHandle, PA_PluginParameters params, char * format, long compression, short index, short tNum, short fNum)
{
	long returnValue = 0;
		
#if VERSIONWIN
		IStream* pIStream    = NULL;
		if(CreateStreamOnHGlobal(NULL, TRUE, (LPSTREAM*)&pIStream) == S_OK)
		{
			CLSID pngClsid;
			if (GetEncoderClsid(format, pngClsid) >= 0)
			{ 
				Status SaveStatus = Ok;
				if (!strcmp(format, "jpg"))
				{
					EncoderParameters* 	encoderParameters = (EncoderParameters*) malloc(sizeof(EncoderParameters));
					encoderParameters->Count = 1;
					encoderParameters->Parameter[0].Guid = EncoderQuality;
					encoderParameters->Parameter[0].Type = EncoderParameterValueTypeLong;
					encoderParameters->Parameter[0].NumberOfValues = 1;
					ULONG  quality = compression;
					encoderParameters->Parameter[0].Value = &quality;

					SaveStatus = AreaHandle->image->Save(pIStream, &pngClsid, encoderParameters);
					free(encoderParameters);
				}
				else
					SaveStatus = AreaHandle->image->Save(pIStream, &pngClsid, NULL);
				if (SaveStatus == Ok)
				{
					ULARGE_INTEGER ulnSize;
					LARGE_INTEGER lnOffset;
					lnOffset.QuadPart = 0;
					pIStream->Seek(lnOffset, STREAM_SEEK_END, &ulnSize);
					pIStream->Seek(lnOffset, STREAM_SEEK_SET, NULL);
					char *pBuff = new char[(int) ulnSize.QuadPart];
					ULONG ulBytesRead;
					if(pIStream->Read(pBuff, (int) ulnSize.QuadPart, &ulBytesRead) == S_OK)
						if ((tNum) && (fNum))
							PA_SetBlobField(tNum, fNum, pBuff, (int) ulnSize.QuadPart);  
						else
							PA_SetBlobParameter (params, index, pBuff, (int) ulnSize.QuadPart);  
					else
						returnValue = kError_Thumbnail;
					delete pBuff;
				}
				else
					returnValue = kError_Thumbnail;
			}
			else
				returnValue = kError_Thumbnail;
			pIStream->Release();
		}
		else
			returnValue = kError_Thumbnail;
#else
			GraphicsExportComponent exporter;
			Handle	exportBlob = NewHandle(0);
			
			OSErr err = OpenADefaultComponent( GraphicsExporterComponentType,  GetEncoderType(format),   &exporter );  
			
			err = GraphicsExportSetInputPicture (exporter, AreaHandle->image);
			if (!err)
				err = GraphicsExportSetOutputHandle (exporter,  exportBlob);
			if ((!err) && (compression)	)
				err = GraphicsExportSetCompressionQuality( exporter,  GetCompression( compression) );
			if (!err)
			{	err = GraphicsExportDoExport (exporter, NULL);
			if (!err)
				{
					if((tNum) && (fNum))
						PA_SetBlobHandleField(tNum, fNum, exportBlob);
					else
						PA_SetBlobHandleParameter(params, index, exportBlob);
					exportBlob = NULL;
				}
			}
			if (exportBlob)
				DisposeHandle(exportBlob);
				
			 CloseComponent ( exporter );	
			 returnValue = err;

#endif		
	
	return returnValue ;
}


void SavePictureBlob( PA_PluginParameters params )
{
	char format[255];
	long returnValue = 0;
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		PA_GetStringParameter(params, 3, format);
		long compression =PA_GetLongParameter(params, 4);

		returnValue = SavePictureBlob(AreaHandle, params, format, compression, 2, 0, 0);
		}
	else
		returnValue = kError_unknownArea;
	
	PA_ReturnLong( params, returnValue );
}



void Save4DPicture( PA_PluginParameters params)
{
	ImageAreaStruct *AreaHandle;
	long returnValue = 0;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	
		returnValue = Save4DPicture( AreaHandle, params, 2, 0, 0, 0);
	}
	else
		returnValue = kError_unknownArea;

	PA_ReturnLong( params, returnValue );
}


// if params/index is passed, it returns the picture as plugin parameter
// if tNum/fNum is passed, it returns the picture as field
// if UsePictVariable = 1, it does not delete the variable PCContainerInternalPict, so it can be used outside

#if VERSIONWIN
long Save4DPicture( ImageAreaStruct *AreaHandle, PA_PluginParameters params, short index, short tNum, short fNum, char UsePictVariable)
{
	long returnValue = 0;

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		IStream* pIStream    = NULL;
		if(CreateStreamOnHGlobal(NULL, TRUE, (LPSTREAM*)&pIStream) == S_OK)
		{
			CLSID pngClsid;
			GetEncoderClsid(L"image/bmp", &pngClsid);
			Status SaveStatus = AreaHandle->image->Save(pIStream, &pngClsid, NULL);
			if (SaveStatus == Ok)
			{
				ULARGE_INTEGER ulnSize;
				LARGE_INTEGER lnOffset;
				lnOffset.QuadPart = 0;
				pIStream->Seek(lnOffset, STREAM_SEEK_END, &ulnSize);
				pIStream->Seek(lnOffset, STREAM_SEEK_SET, NULL);
				char *pBuff = new char[(int) ulnSize.QuadPart];
				ULONG ulBytesRead;
				if(pIStream->Read(pBuff, (int) ulnSize.QuadPart, &ulBytesRead) == S_OK)
					returnValue = BMPtoMacPictParam(pBuff, (int) ulnSize.QuadPart, params, index, tNum, fNum, UsePictVariable);  
				else
					returnValue = kError_Thumbnail;
				delete pBuff;
			}
			else
				returnValue = kError_Thumbnail;
			pIStream->Release();
		}
		else
			returnValue = kError_Thumbnail;	
		}
	else
		returnValue = kError_unknownArea;	
	return returnValue;
}
#else   //  Mac Version, last Parameter never used!
long Save4DPicture( ImageAreaStruct *AreaHandle, PA_PluginParameters params, short index, short tNum, short fNum, char UsePictVariable)
{
	long returnValue = 0;

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		PA_PictureInfo info = {0, 0, 0};
		if((tNum) && (fNum))
			PA_SetPictureField (tNum, fNum, PA_LockHandle((PA_Handle) AreaHandle->image), PA_GetHandleSize((PA_Handle) AreaHandle->image), info);
		else		
			PA_SetPictureParameter (params, index, PA_LockHandle((PA_Handle) AreaHandle->image), PA_GetHandleSize((PA_Handle) AreaHandle->image), info);
		PA_UnlockHandle((PA_Handle) AreaHandle->image);	
		}
	else
		returnValue = kError_unknownArea;
	
	return returnValue;
}
#endif	

// to be called as menu command
void ClearPicture( PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	ClearImage(AreaHandle);
		AreaHandle->RedrawRequired = true;
	}
}

// to be called from plugin area / edit menu
void ClearPictureArea( PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	ClearImage(AreaHandle);
		// AreaHandle->RedrawRequired = true;
#if VERSIONWIN
		  RECT pRect;
		  ConvertPA_RectToRect(AreaHandle->oldSize, &pRect);
		  InvalidateRect(AreaHandle->FormWindow, &pRect, FALSE);
#else
		InvalWindowRect(AreaHandle->FormWindow, (const Rect*) &AreaHandle->oldSize);
#endif
		AreaHandle->Dirty = true;
	}
}

void CopyPictureArea( PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	long err = 0;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	// get's pict of current area and put it with Execute in Clipboard
#if VERSIONWIN
		// convert image into Pic_Handle, return as 4D picture variable named PCContainerInternalPict
		err = Save4DPicture( AreaHandle, NULL, 0, 0, 0, 1);
#else
		// create new 4D picture variable named PCContainerInternalPict from Pic_Handle
		PA_Variable aPicture = PA_CreateVariable(eVK_Picture, 0);
		PA_PictureInfo info = {0, 0, 0};
		PA_SetPictureVariable (&aPicture, PA_LockHandle((PA_Handle)   AreaHandle->image), PA_GetHandleSize((PA_Handle) AreaHandle->image), info);
		err = PA_GetLastError();
		PA_UnlockHandle( (PA_Handle) AreaHandle->image );
		PA_SetVariable ("PCContainerInternalPict", aPicture, 1);
		
#endif

		if (!err)
		{
		  char Command[255], DoCommand[255];
		  MacGetIndString(Command, 8, 521);  // set picture to clipboard
		  strlcpy(DoCommand, Command, 255);
		  strlcat(DoCommand, "(PCContainerInternalPict)", 255);
		  PA_ExecuteMethod(DoCommand, strlen(DoCommand));

		  MacGetIndString(Command, 8, 89);  // clear variable
		  strlcpy(DoCommand, Command, 255);
		  strlcat(DoCommand, "(PCContainerInternalPict)", 255);
		  PA_ExecuteMethod(DoCommand, strlen(DoCommand));  
		  
		//  PA_ClearVariable(&aPicture);

		}
	}
}

void PastePictureArea( PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	long err = 0;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	// get's pict from clipboard and assign it to area

		  PA_Variable aPicture = PA_CreateVariable(eVK_Picture, 0);
		  PA_SetVariable ("PCContainerInternalPict", aPicture, 1);
		  // PA_ClearVariable(&aPicture);
		  char Command[255], DoCommand[255];
		  MacGetIndString(Command, 8, 522);  // 522	GET PICTURE FROM CLIPBOARD (picture)
		  strlcpy(DoCommand, Command, 255);
		  strlcat(DoCommand, " (PCContainerInternalPict)", 255);
		  PA_ExecuteMethod(DoCommand, strlen(DoCommand));
		  aPicture = PA_GetVariable("PCContainerInternalPict");

		  ClearImage(AreaHandle);

		  PA_PictureInfo   theInfo;
		  PA_Handle newImage = PA_GetPictureHandleVariable (aPicture, &theInfo);

		  // set variable to area
		  err = Load4DPictureInHandle(AreaHandle, newImage);

		  // old variable not needed anymore
		  MacGetIndString(Command, 8, 89);  // clear variable
		  strlcpy(DoCommand, Command, 255);
		  strlcat(DoCommand, " (PCContainerInternalPict)", 255);
		  // PA_ClearVariable(&aPicture);

		  AreaHandle->Dirty = true; 
#if VERSIONWIN
		  RECT pRect;
		  ConvertPA_RectToRect(AreaHandle->oldSize, &pRect);
		  InvalidateRect(AreaHandle->FormWindow, &pRect, FALSE);
#else
		  InvalWindowRect(AreaHandle->FormWindow, (const Rect*) &AreaHandle->oldSize);
#endif
	}
}

void SetEditMenu(PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	PA_Variable      aLongint;
	char DoCopy, DoPaste, DoClear, DoCut;
	char DoCommand[255], Command[255];
	long CountPict = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	// get's pict from clipboard and assign it to area
		MacGetIndString(Command, 8, 400);  // 400	Test clipboard (dataType) --> Number
		strlcpy(DoCommand, Command, 255);
		strlcat(DoCommand, " (\"PICT\")", 255);
		aLongint = PA_ExecuteFunction(DoCommand, strlen(DoCommand));
		if (PA_GetVariableKind (aLongint) == eVK_Longint)
			CountPict = PA_GetLongintVariable(aLongint);
		else
			if (PA_GetVariableKind (aLongint) == eVK_Real)
				CountPict = (long) PA_GetRealVariable(aLongint);
		PA_ClearVariable(&aLongint);

		if ((AreaHandle->image) && (AreaHandle->AllowCopy))
			DoCopy = 1;
		else
			DoCopy=0;
		
		if ((CountPict) && (AreaHandle->AllowPaste))
			DoPaste = 1; 
		else DoPaste = 0;
		
		if ((AreaHandle->image) && (AreaHandle->AllowPaste))
			DoClear = 1;
		else
			DoClear=0;

		if ((AreaHandle->image) && (AreaHandle->AllowCopy) && (AreaHandle->AllowPaste))
			DoCut = 1;
		else
			DoCut=0;

		PA_UpdateEditMenu("Undo not possible", 0, 0, DoCopy, DoCut, DoPaste, DoClear, 0);
	}
}


long LoadBlobInHandle( ImageAreaStruct *AreaHandle, PA_Handle theBlob )
{
	long returnValue = 0;

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) && (theBlob != NULL))
	{	
		ClearImage(AreaHandle);
#if VERSIONWIN	
		 long size = PA_GetHandleSize(theBlob);
		 HGLOBAL hGlobal = GlobalAlloc( GMEM_MOVEABLE, size );
		 LPVOID pvData = GlobalLock( hGlobal );
		 PA_MoveBlock(PA_LockHandle(theBlob), pvData, size);
		 GlobalUnlock(hGlobal);
		 AreaHandle->pStream = NULL;
		 HRESULT hr = CreateStreamOnHGlobal( hGlobal, TRUE, &AreaHandle->pStream );

		 AreaHandle->image = Image::FromStream(AreaHandle->pStream, false);
		returnValue = AreaHandle->image->GetLastStatus();
#else
		GraphicsImportComponent gi = 0;
		// Technical Q&A ATMTB56 reading pictures from memory http://developer.apple.com/qa/qtmtb/qtmtb56.html
		// http://developer.apple.com/technotes/tn/tn1195.html
		Handle dataRef=NULL;
		returnValue = PtrToHand( &theBlob, &dataRef, sizeof(Handle));
		if (dataRef)
			{
			returnValue = GetGraphicsImporterForDataRef (dataRef, HandleDataHandlerSubType, &gi);  // -1856
			DisposeHandle(dataRef);
			if (!returnValue)
			{	StorePictureInformations(AreaHandle, gi);
				returnValue = GraphicsImportGetAsPicture(gi, &AreaHandle->image);
				}
			CloseComponent( gi);
			}
			
#endif
		AreaHandle->RedrawRequired = true;
	}
	else
		returnValue = kError_unknownArea;
	return returnValue;
}

void LoadPictureBlob( PA_PluginParameters params )
{
	long returnValue = 0;
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		returnValue = LoadBlobInHandle(AreaHandle, PA_GetBlobHandleParameter (params, 2));	
	else
		returnValue = kError_unknownArea;
	PA_ReturnLong( params, returnValue );
}

long Load4DPictureInHandle(ImageAreaStruct *AreaHandle, PA_Handle thePict )
// thePict does belong to the outside call, the outside method is supposed to delete it if not needed anymore
{
	long returnValue = 0;
	
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag) && (thePict != NULL))
		{	
		ClearImage(AreaHandle);
#if VERSIONWIN
		HENHMETAFILE EMFPicturehandle=0;
		
		EMFPicturehandle =(HENHMETAFILE ) PA_PictureToEMF (PA_LockHandle(thePict), PA_GetHandleSize(thePict))  ;
		PA_UnlockHandle(thePict);
		if (EMFPicturehandle)
			{
			 UINT size = GetEnhMetaFileBits(EMFPicturehandle, 0, NULL); 
			 HGLOBAL hGlobal = GlobalAlloc( GMEM_MOVEABLE, size );
			 LPVOID pvData = GlobalLock( hGlobal );
			 size = GetEnhMetaFileBits(EMFPicturehandle, size, (LPBYTE) pvData); 
			 GlobalUnlock(hGlobal);
			 AreaHandle->pStream = NULL;
			 HRESULT hr = CreateStreamOnHGlobal( hGlobal, TRUE, &AreaHandle->pStream );

			 AreaHandle->image = Image::FromStream(AreaHandle->pStream, false);
			 returnValue = AreaHandle->image->GetLastStatus();

			DeleteEnhMetaFile(EMFPicturehandle);
			}
#else  
		//Mac
		long size = PA_GetHandleSize(thePict);
		AreaHandle->image = (PicHandle) PA_NewHandle(size);
		if (AreaHandle->image)
			{	PA_LockHandle((Handle) AreaHandle->image);
				PA_LockHandle((Handle) thePict);
				PA_MoveBlock(*thePict, *AreaHandle->image, size);
				PA_UnlockHandle((Handle) AreaHandle->image);
				PA_UnlockHandle((Handle) thePict);
				StorePictureInformations(AreaHandle, NULL);
			}
		else
			returnValue = -108;
#endif
		AreaHandle->RedrawRequired = true;
		}
	else
		returnValue = kError_unknownArea;
	return returnValue;
}

void Load4DPicture(PA_PluginParameters params )
{
	ImageAreaStruct *AreaHandle;
	long returnValue = 0;
	PA_PictureInfo Picture_info;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		returnValue = Load4DPictureInHandle(AreaHandle, PA_GetPictureHandleParameter(params, 2, &Picture_info));
		PA_ReturnLong( params, returnValue );
		}
	else
		PA_ReturnLong(params, kError_unknownArea);
}

void ClearImage(ImageAreaStruct *AreaHandle )
{
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	
		AreaHandle->ZoomX = AreaHandle->ZoomY = 0;
#if VERSIONWIN	
		if(AreaHandle->pStream != NULL)
		{
			AreaHandle->pStream->Release();
			AreaHandle->pStream = NULL;
		}
		if(AreaHandle->image != NULL)
			{	delete AreaHandle->image;
			AreaHandle->image = NULL; 
			}
#else
		if (AreaHandle->image)
			{	DisposeHandle((Handle) AreaHandle->image);
			AreaHandle->image = 0L;
			}
#endif
	}
}


void SetDisplayProperties( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	long returnValue = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	AreaHandle->ZoomX = PA_GetLongParameter( params, 2 );
		AreaHandle->ZoomY = PA_GetLongParameter( params, 3 );
		AreaHandle->ScrollX = PA_GetLongParameter( params, 4 );
		AreaHandle->ScrollY = PA_GetLongParameter( params, 5 );
		// Control will be reset with next redraw, no need to do anything here

		AreaHandle->RedrawRequired = true;
	}
	else
		returnValue = kError_unknownArea;

	PA_ReturnLong( params, returnValue );
}

void GetDisplayProperties( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	long returnValue = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	PA_SetLongParameter( params, 2 , AreaHandle->ZoomX);
		PA_SetLongParameter( params, 3 , AreaHandle->ZoomY);
		PA_SetLongParameter( params, 4 , AreaHandle->ScrollX);
		PA_SetLongParameter( params, 5 , AreaHandle->ScrollY);
	}
	else
		returnValue = kError_unknownArea;
	PA_ReturnLong( params, returnValue );
}

void GetPictureProperties( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	long returnValue = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	{	if(AreaHandle->image != NULL)
		{
#if VERSIONWIN
			PA_SetLongParameter( params, 2 , (long) AreaHandle->image->GetHorizontalResolution());
			PA_SetLongParameter( params, 3 , (long) AreaHandle->image->GetVerticalResolution());
			PA_SetLongParameter( params, 4 , AreaHandle->image->GetWidth());
			PA_SetLongParameter( params, 5 , AreaHandle->image->GetHeight());

			GUID guid;
			AreaHandle->image->GetRawFormat(&guid);
			//  switch (guid)  // not allowed, too bad...
			PA_SetStringParameter( params, 6, "Unknown");
			if(guid == ImageFormatJPEG)
				PA_SetStringParameter( params, 6, "JPEG" );
			if(guid == ImageFormatMemoryBMP)
				PA_SetStringParameter( params, 6, "BMP" );
			if(guid == ImageFormatBMP)
				PA_SetStringParameter( params, 6, "BMP" );
			if(guid == ImageFormatEMF)
				PA_SetStringParameter( params, 6, "EMF" );
			if(guid == ImageFormatWMF)
				PA_SetStringParameter( params, 6, "WMF" );
			if(guid == ImageFormatPNG)
				PA_SetStringParameter( params, 6, "PNG" );
			if(guid == ImageFormatGIF)
				PA_SetStringParameter( params, 6, "GIF");
			if(guid == ImageFormatTIFF)
				PA_SetStringParameter( params, 6, "TIF" );
			if(guid == ImageFormatIcon)
				PA_SetStringParameter( params, 6, "ICON");
			returnValue = AreaHandle->image->GetLastStatus();
#else
			if (AreaHandle->PictWidth == 0)
				{	// picture was received as PicHandle
					PicHandle thePicture = AreaHandle->image;
					PA_SetLongParameter( params, 2 , 0);
					PA_SetLongParameter( params, 3 , 0);
					PA_SetLongParameter( params, 4 , (*thePicture)->picFrame.right - (*thePicture)->picFrame.left);
					PA_SetLongParameter( params, 5 , (*thePicture)->picFrame.bottom - (*thePicture)->picFrame.top);
					PA_SetStringParameter( params, 6, "PICT" );
				}
			else
				{
				PA_SetLongParameter( params, 2 , (long) AreaHandle->PictResX);
				PA_SetLongParameter( params, 3 , (long) AreaHandle->PictResY);
				PA_SetLongParameter( params, 4 , AreaHandle->PictWidth);
				PA_SetLongParameter( params, 5 , AreaHandle->PictHeight);
				PA_SetStringParameter( params, 6, (char *) AreaHandle->Format );
				}
					
#endif
		}
	}
	else
		returnValue = kError_unknownArea;

	PA_ReturnLong( params, returnValue );
}

void GetAreaModified(PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	long returnValue = 0;

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if(AreaHandle->Dirty)
			returnValue = 1;
	
	PA_ReturnLong( params, returnValue );
}

void AreaAllowNextDrop(PA_PluginParameters params )
{	//  Allows to change behavior of next drop, to accept a normally not accepted or to disallow a normally accepted drop
	ImageAreaStruct * AreaHandle;

	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	long Accept = PA_GetLongParameter( params, 2 );

	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if(Accept)
			AreaHandle->AllowNextDrop = 1;
		else
			AreaHandle->AllowNextDrop = 0;
}


#if VERSIONWIN
// if params is passed, it returns the result as paramter set by index
// if tNum/fNum is passed, it returns the result as field
// if UsePictVariable = 1, it does not delete the variable PCContainerInternalPict, so it can be used outside
long BMPtoMacPictParam(char *pBuff, int theSize, PA_PluginParameters params, short index, short tNum, short fNum, char UsePictVariable)
{ // AP Read Picture BLOB (file; image)  Longint
	long  returnValue = 0;
	
	PA_Variable      aLongint;
	char*			 testmeth = "AP Toolbar installed";
	aLongint = PA_ExecuteFunction(testmeth, strlen(testmeth));
	if (PA_GetVariableKind (aLongint) == eVK_Undefined)
	{
		returnValue = kError_4DPack;  // 4D Pack not installed
		PA_ClearVariable(&aLongint);	
	}
	else
	{  
		PA_ClearVariable(&aLongint);
		PA_Variable  aBlob = PA_CreateVariable(eVK_Blob, 0);
		PA_SetBlobVariable (&aBlob, pBuff, theSize);
		PA_Variable aPicture = PA_CreateVariable(eVK_Picture, 0);
		PA_SetVariable ("PCContainerInternalBlob", aBlob, 1);
		PA_SetVariable ("PCContainerInternalPict", aPicture, 1);
		char*  DoCommand = "AP Read Picture BLOB (PCContainerInternalBlob; PCContainerInternalPict)";
		aLongint = PA_ExecuteFunction(DoCommand, strlen(DoCommand));
		// if error return error code, else read picture and return that. Clear blob and pict variable
		returnValue = PA_GetLongintVariable (aLongint);
		if (!returnValue)
		{
			      PA_PictureInfo   info;
				  PA_Handle thePict = PA_GetPictureHandleVariable (aPicture, &info) ;
				  long pictSize = PA_GetHandleSize(thePict);
				  if ((tNum) && (fNum))
					PA_SetPictureField(tNum, fNum, PA_LockHandle(thePict), pictSize, info);
				  else
					  if (params)
							PA_SetPictureParameter (params, index, PA_LockHandle(thePict), pictSize, info);

				  PA_UnlockHandle(thePict);
				  char Command[255], DoCommand[255];
				  MacGetIndString(Command, 8, 89);  // clear variable
				  strlcpy(DoCommand, Command, 255);
				  strlcat(DoCommand, " (PCContainerInternalBlob)", 255);
				  PA_ExecuteMethod(DoCommand, strlen(DoCommand));

				  if (UsePictVariable == 0)
				  {
					strlcpy(DoCommand, Command, 255);
					strlcat(DoCommand, " (PCContainerInternalPict)", 255);
					PA_ExecuteMethod(DoCommand, strlen(DoCommand));
				  }
		}
		PA_ClearVariable(&aLongint);
	}
	return returnValue;						
}
#endif

void GetThumbnail( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	long returnValue = 0;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	if(AreaHandle->image != NULL)
			{
#if VERSIONWIN
			long result=0;
			{
			long thumbsize = PA_GetLongParameter( params, 3 );
			if (thumbsize == 0) thumbsize = 48;
			double scale;
			double pictx = AreaHandle->image->GetWidth();
			double picty = AreaHandle->image->GetHeight();
			if (pictx > picty)
				scale = pictx / thumbsize;
			else
				scale = picty / thumbsize;
			pictx /= scale;
			picty /= scale;
			Image* pThumbnail = AreaHandle->image->GetThumbnailImage((int) pictx, (int) picty, NULL, NULL);
			IStream* pIStream    = NULL;
			if(CreateStreamOnHGlobal(NULL, TRUE, (LPSTREAM*)&pIStream) == S_OK)
			{
				CLSID pngClsid;

				GetEncoderClsid(L"image/bmp", &pngClsid);
				Status SaveStatus = pThumbnail->Save(pIStream, &pngClsid, NULL);
				if (SaveStatus == Ok)
				{
					ULARGE_INTEGER ulnSize;
					LARGE_INTEGER lnOffset;
					lnOffset.QuadPart = 0;
					pIStream->Seek(lnOffset, STREAM_SEEK_END, &ulnSize);
					pIStream->Seek(lnOffset, STREAM_SEEK_SET, NULL);
//					{
						char *pBuff = new char[(int) ulnSize.QuadPart];
						ULONG ulBytesRead;
						if(pIStream->Read(pBuff, (int) ulnSize.QuadPart, &ulBytesRead) == S_OK)
							returnValue = BMPtoMacPictParam(pBuff, (int) ulnSize.QuadPart, params, 2, 0, 0, false);  
						else
							returnValue = kError_Thumbnail;
						delete pBuff;
				}
				else
					returnValue = kError_Thumbnail;
				pIStream->Release();
			}
			else
				returnValue = kError_Thumbnail;
			delete pThumbnail;
			}
#else			
			PicHandle mypict = (PicHandle) PA_NewHandle(0);
			returnValue = MakeThumbnailFromPicture (AreaHandle->image, 0,  mypict , NULL);		
			
			PA_PictureInfo info = {0, 0, 0};
			PA_SetPictureHandleParameter(params, 2, (PA_Handle) mypict, info);
			
#endif
			}
		}
	else
		returnValue = kError_unknownArea;
	PA_ReturnLong( params, returnValue );
}

void SetAreaProperties(PA_PluginParameters params)
{
	ImageAreaStruct * AreaHandle;	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
			if(AreaHandle->image != NULL)
			{
			AreaHandle->AllowCopy = PA_GetLongParameter( params, 2 );
			AreaHandle->AllowPaste = PA_GetLongParameter( params, 3 );
			AreaHandle->AutoDrop = PA_GetLongParameter( params, 4 );
			}	
}
			
			// ------------------------------- --------------- ------------------------------
// ------------------------------- --------------- ------------------------------
// ------------------------------- Other Utilities ------------------------------

#if VERSIONWIN

void DoScrollAction(ImageAreaStruct *AreaHandle,  HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	SCROLLINFO si;
	int yMaxScroll;
	int xDelta = 0, yDelta = 0, yNewPos;  
	RECT scrollrect;

	HWND scrollbar;

	scrollbar = (HWND) lParam;

	xDelta = 0;
	si.cbSize = sizeof(si); 
	si.fMask  = SIF_RANGE  | SIF_PAGE   | SIF_POS; 

	GetScrollInfo(scrollbar, SB_CTL, &si);

	yMaxScroll = si.nMax;
		
	switch (LOWORD(wParam)) 
	{ 
		// User clicked the shaft above the scroll box. 		
		case SB_PAGEUP: 
			yNewPos = si.nPos - si.nPage; 
			break; 

		// User clicked the shaft below the scroll box. 			
		case SB_PAGEDOWN: 
			yNewPos = si.nPos + si.nPage; 
			break; 

		// User clicked the top arrow. 			
		case SB_LINEUP: 
			yNewPos = si.nPos - 1; 
			break; 

		// User clicked the bottom arrow. 			
		case SB_LINEDOWN: 
			yNewPos = si.nPos + 1; 
			break; 

		// User dragged the scroll box. 			
		case SB_THUMBPOSITION: 
			yNewPos = HIWORD(wParam); 
			break; 

		case SB_THUMBTRACK:  //  live scrolling
			yNewPos = HIWORD(wParam); 
			break; 

		default: 
			yNewPos = si.nPos; 
	} 

	// New position must be between 0 and the screen height. 			
	yNewPos = max(0, yNewPos); 
	yNewPos = min((UINT) yMaxScroll-si.nPage, (UINT) yNewPos); // 17.4.04 - Scrollbar by Thumb bad max
			
	if (yNewPos == si.nPos) // If the current position does not change, do not scroll.		
		return;

	if (uMsg == WM_VSCROLL)
		yDelta = yNewPos - si.nPos; // Determine the amount scrolled (in pixels). 	
	else
		xDelta = yNewPos - si.nPos;
			
	// Reset the scroll bar. 
	si.cbSize = sizeof(si); 
	si.fMask  = SIF_POS; 
	si.nPos   = yNewPos; 
	if (uMsg == WM_VSCROLL)
	{	SetScrollInfo(scrollbar, SB_CTL, &si, TRUE); 
		AreaHandle->ScrollY = yNewPos; 
	}
	else
	{	SetScrollInfo(scrollbar, SB_CTL, &si, TRUE); 
		AreaHandle->ScrollX = yNewPos; 
	}

	ConvertPA_RectToRect(AreaHandle->oldSize, &scrollrect);
	if (AreaHandle->HorScroll && AreaHandle->HScrollActive)
		scrollrect.bottom -= AreaScrollbarSize;
	if (AreaHandle->VertScroll && AreaHandle->VScrollActive)
		scrollrect.right -= AreaScrollbarSize;

	ScrollWindowEx(hwnd, -xDelta, -yDelta, &scrollrect, 
			&scrollrect, (HRGN) NULL, (LPRECT) NULL, 
			SW_INVALIDATE  ); 
	UpdateWindow(hwnd); 

}

LRESULT CALLBACK ScrollWndProc(HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)   
{ 
	HANDLE winprop;
	
	if ((uMsg == WM_HSCROLL) || (uMsg == WM_VSCROLL))
	{	winprop = GetProp((HWND) lParam, "ScrollAreaHandle"); 
		if (winprop != 0)  //  it's our scrollbar!
		{	
			DoScrollAction(( ImageAreaStruct *) winprop, hwnd, uMsg, wParam, lParam);
			return 0;
		}
	}

	winprop = GetProp(hwnd, "GDIScrollHandler");

	if (winprop != 0)
		return CallWindowProc((WNDPROC) winprop, hwnd, uMsg, wParam, lParam); 
	else
	{	// oops, this is not supposed to happen, major bug!
		#ifdef _DEBUG
			PA_Alert("we have an error in ScrollWndProc");
		#endif
		return DefWindowProc(hwnd, uMsg, wParam, lParam);   // better than nothing...
	}
}
#else  // Mac Control Action for scrollbar

void MyScrollbarActionProc(ControlRef theControl, ControlPartCode partCode)
{
	int xDelta = 0, yDelta = 0, yNewPos;  

	int yMaxScroll = GetControl32BitMaximum(theControl);
	int yOldPos = GetControl32BitValue(theControl);
	int yView = GetControlViewSize(theControl);
	
	switch (partCode)
		{
		case kControlUpButtonPart:
			yNewPos = yOldPos - 1;
			break;

		case kControlDownButtonPart:
			yNewPos = yOldPos + 1;
			break;

		case kControlPageUpPart:
			yNewPos = yOldPos - yView;
			break;
			
		case kControlPageDownPart:
			yNewPos = yOldPos + yView;
			break;	
			
		case kControlIndicatorPart:
			yNewPos = yOldPos;
			
		default: yNewPos = yOldPos;
		}
	
	yNewPos = max(0, yNewPos); 
	yNewPos = min(yMaxScroll, yNewPos); // on Mac maxpos = pict size, 

	SetControl32BitValue(theControl,yNewPos);
	ImageAreaStruct *AreaHandle = (ImageAreaStruct *) GetControlReference(theControl);
	if (AreaHandle)
		{  // for Thumb Dragging we do not get a valid handle, so this is not working here...
		if (theControl == AreaHandle->HorScroll)
			{	
			xDelta = yNewPos - AreaHandle->ScrollX;
			yDelta = 0;
			AreaHandle->ScrollX = yNewPos;
			}
		if (theControl == AreaHandle->VertScroll)
			{	
			yDelta = yNewPos - AreaHandle->ScrollY;
			xDelta = 0;
			AreaHandle->ScrollY = yNewPos;
			}
		DoMacScroll(AreaHandle, xDelta, yDelta);
		}
}

void DoMacScroll(ImageAreaStruct *AreaHandle, short xDelta, short yDelta)
{
	long SizeX, SizeY;
	if (xDelta || yDelta)
		{  // do nothing if position did not changed
		RgnHandle oldRgn = NewRgn();
		RgnHandle updateRgn = NewRgn();
		GetClip(oldRgn);
		PA_Rect TheScrollRect = AreaHandle->oldSize;
		if (AreaHandle->VScrollActive)
			TheScrollRect.fRight = TheScrollRect.fRight - AreaScrollbarSize ;
		if (AreaHandle->HScrollActive)
			TheScrollRect.fBottom = TheScrollRect.fBottom - AreaScrollbarSize;
			
		// we only scroll the used part. Not sure if there is a speed difference, but else it draw a white background for scrolling.
		GetScaledImageSize(AreaHandle, SizeX, SizeY, TheScrollRect.fRight-TheScrollRect.fLeft, TheScrollRect.fBottom-TheScrollRect.fTop);
		SizeX += TheScrollRect.fLeft;
		TheScrollRect.fRight = min(TheScrollRect.fRight, SizeX);
		SizeY += TheScrollRect.fTop;
		TheScrollRect.fBottom = min(TheScrollRect.fBottom, SizeY);
		
		ScrollRect((const Rect *) &TheScrollRect, -xDelta, -yDelta, updateRgn);
		SetClip(updateRgn);
		// redraw picture
		PA_Rect myrect = AreaHandle->oldSize;
		long SizeX, SizeY;
		GetScaledImageSize(AreaHandle, SizeX, SizeY, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
		DrawImageSub_Mac(AreaHandle, myrect, SizeX, SizeY);
		SetClip(oldRgn);
		DisposeRgn(oldRgn);
		DisposeRgn(updateRgn);
		}
	
}
#endif

#if VERSIONWIN
void AreaWheel( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	SCROLLINFO si;
	int yMaxScroll;
	int xDelta = 0, yDelta = 0, yNewPos;  
	RECT scrollrect;
	short mouseincr = -PA_GetMouseWheelIncrement( params );
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if (AreaHandle->VertScroll)
		{	
			si.cbSize = sizeof(si); 
			si.fMask  = SIF_RANGE  | SIF_PAGE   | SIF_POS; 
			GetScrollInfo(AreaHandle->VertScroll, SB_CTL, &si);

			yMaxScroll = si.nMax;
			yNewPos = (mouseincr/3) * (si.nPage /3);
			yNewPos = si.nPos + yNewPos;
		
			// New position must be between 0 and the screen height. 			
			yNewPos = max(0, yNewPos); 
			yNewPos = min((UINT) yMaxScroll-si.nPage, (UINT) yNewPos);
					
			if (yNewPos != si.nPos) // If the current position does not change, do not scroll.		
			{
				yDelta = yNewPos - si.nPos; // Determine the amount scrolled (in pixels). 	
						
				// Reset the scroll bar. 
				si.cbSize = sizeof(si); 
				si.fMask  = SIF_POS; 
				si.nPos   = yNewPos; 
				SetScrollInfo(AreaHandle->VertScroll, SB_CTL, &si, TRUE); 
				AreaHandle->ScrollY = yNewPos; 

				ConvertPA_RectToRect(AreaHandle->oldSize, &scrollrect);
				if (AreaHandle->HorScroll && AreaHandle->HScrollActive)
					scrollrect.bottom -= AreaScrollbarSize;
				if (AreaHandle->VertScroll && AreaHandle->VScrollActive)
					scrollrect.right -= AreaScrollbarSize;

				ScrollWindowEx(GetParent(AreaHandle->VertScroll), -xDelta, -yDelta, &scrollrect, 
						&scrollrect, (HRGN) NULL, (LPRECT) NULL, 
						SW_INVALIDATE  ); 
				UpdateWindow(GetParent(AreaHandle->VertScroll)); 
			}
		}
}
#else
void AreaWheel( PA_PluginParameters params )
{
	ImageAreaStruct * AreaHandle;
	int mouseincr = -PA_GetMouseWheelIncrement( params );
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		if (AreaHandle->VertScroll)
		{	
		int yMaxScroll = GetControl32BitMaximum(AreaHandle->VertScroll);
		int yView = GetControlViewSize(AreaHandle->VertScroll);
		int yOldPos = GetControl32BitValue(AreaHandle->VertScroll);

		mouseincr = (mouseincr/3) * (yView/6);
		int yNewPos = yOldPos + mouseincr;

		yNewPos = max(0, yNewPos); 
		yNewPos = min(yMaxScroll, yNewPos); // on Mac maxpos = pict size, 
		if (yNewPos != AreaHandle->ScrollY)
			{
			AreaHandle->ScrollY = yNewPos;
			SetControl32BitValue(AreaHandle->VertScroll, AreaHandle->ScrollY);
			mouseincr = yNewPos-yOldPos;
			DoMacScroll(AreaHandle, 0, mouseincr);		
			}
		}	
}
#endif



#if VERSIONWIN

void DrawImage(ImageAreaStruct *AreaHandle, HDC hdc, PA_Rect myrect)
{
	long OffsetX, OffsetY;
	long SizeX, SizeY;
	bool NewHShow, NewVShow;

	SCROLLINFO si;
	int yMaxScroll, Height, yNewSize; 

	Graphics graphics(hdc);

		if(AreaHandle->image != NULL)
		{	graphics.SetInterpolationMode((InterpolationMode) InterpolationModeDefault);

			// scrollbar handling, do nothing without scrollbars
			if (AreaHandle->HorScroll && AreaHandle->VertScroll)
			{
				// scrollbar - calculate if visible
				if ((AreaHandle->ZoomX <= 0) || (AreaHandle->ZoomY <= 0))  // automatic modus - hide them
					NewHShow = NewVShow = false;
				else
				{  // show if pict larger than space
					if ((!AreaHandle->HScrollShow) && (!AreaHandle->VScrollShow))
						NewHShow = NewVShow = false;
					else
					{	NewHShow = NewVShow = false;
						GetScaledImageSize(AreaHandle, SizeX, SizeY, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
						if (SizeX > (myrect.fRight - myrect.fLeft))
							if (AreaHandle->HorScroll && AreaHandle->HScrollShow)
								NewHShow = true;
						if (SizeY > (myrect.fBottom - myrect.fTop))
							if (AreaHandle->VertScroll && AreaHandle->VScrollShow)
								NewVShow = true;
					}
				}

				// Scrollbar reset
				if (AreaHandle->HorScroll && NewHShow)
				{	// horizontal	
					yNewSize = myrect.fRight - myrect.fLeft;
					if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
						yNewSize -= AreaScrollbarSize;					
					if(AreaHandle->image != NULL)
						Height = SizeX;
					else
						Height = 0;
					yMaxScroll = Height; 
					AreaHandle->ScrollX = min(AreaHandle->ScrollX, (Height-yNewSize));
					AreaHandle->ScrollX = min(AreaHandle->ScrollX, yMaxScroll); 
					si.cbSize = sizeof(si); 
					si.fMask  = SIF_RANGE | SIF_PAGE | SIF_POS; 
					si.nMin   = 0; 
					si.nMax   = yMaxScroll; 
					if ((Height == 0) || (yMaxScroll == 0))
						si.nPage = 0;
					else
						si.nPage = (int) ((double) yNewSize / (double) Height * (double) yMaxScroll);
					if( si.nMax == si.nPage)
						si.nMax = si.nPage = 0;
					si.nPos   = AreaHandle->ScrollX;
					SetScrollInfo(AreaHandle->HorScroll, SB_CTL, &si, FALSE); 
				}
				if (AreaHandle->VertScroll && NewVShow)
				{	// vertical	
					yNewSize = myrect.fBottom - myrect.fTop;
					if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
						yNewSize -= AreaScrollbarSize;	
					if(AreaHandle->image != NULL)
						Height = SizeY;
					else
						Height = 0;
					yMaxScroll = Height; 
					AreaHandle->ScrollY = min(AreaHandle->ScrollY, (Height-yNewSize));
					AreaHandle->ScrollY = min(AreaHandle->ScrollY, yMaxScroll); 
					si.cbSize = sizeof(si); 
					si.fMask  = SIF_RANGE | SIF_PAGE | SIF_POS; 
					si.nMin   = 0; 
					si.nMax   = yMaxScroll; 
					if ((Height == 0) || (yMaxScroll == 0))
						si.nPage = 0;
					else
						si.nPage = (int) ((double) yNewSize / (double) Height * (double) yMaxScroll);
					if( si.nMax == si.nPage)
						si.nMax = si.nPage = 0;
					si.nPos   = AreaHandle->ScrollY;
					SetScrollInfo(AreaHandle->VertScroll, SB_CTL, &si, FALSE); 
				}

				if (AreaHandle->HScrollActive != NewHShow)
				{
					AreaHandle->HScrollActive = NewHShow;
					if (AreaHandle->HScrollActive)
					{	ShowWindow(AreaHandle->HorScroll, SW_SHOWNORMAL); 
						UpdateWindow(AreaHandle->HorScroll);  }
					else
					{	ShowWindow(AreaHandle->HorScroll, SW_HIDE); 
						UpdateWindow(AreaHandle->HorScroll);
					}
				}
				if (AreaHandle->VScrollActive != NewVShow)
				{
					AreaHandle->VScrollActive = NewVShow;
					if (AreaHandle->VScrollActive)
					{	ShowWindow(AreaHandle->VertScroll, SW_SHOWNORMAL); 
						UpdateWindow(AreaHandle->VertScroll); 
					}
					else
					{	ShowWindow(AreaHandle->VertScroll, SW_HIDE); 
						UpdateWindow(AreaHandle->VertScroll); 
					}
				}

				// remove them if visible
				if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
					myrect.fBottom -= AreaScrollbarSize;
				if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
					myrect.fRight -= AreaScrollbarSize;
			}

			GetScaledImageSize(AreaHandle, SizeX, SizeY, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
			
			if ((AreaHandle->ZoomX <= 0) || (AreaHandle->ZoomY <= 0))  // automatic modus
			{
				switch (AreaHandle->ZoomX)
				{	case GDI_ScaledToFit:
						OffsetX = myrect.fLeft;
						OffsetY = myrect.fTop;
						break;

					case GDI_ScaledToFitPropCentered:
						OffsetX = myrect.fLeft + abs((int) ((myrect.fRight-myrect.fLeft)-SizeX) / 2);
						OffsetY = myrect.fTop + abs((int) ((myrect.fBottom-myrect.fTop)-SizeY) / 2); 
						break;

					case GDI_ReduceToFitPropCentered:
						OffsetX = myrect.fLeft + abs((int) ((myrect.fRight-myrect.fLeft)-SizeX) / 2);
						OffsetY = myrect.fTop + abs((int) ((myrect.fBottom-myrect.fTop)-SizeY) / 2); 
						break;

					case GDI_ReduceToFitProp  :
						OffsetX = myrect.fLeft;
						OffsetY = myrect.fTop;
						break;

					// case GDI_ScaledToFitProp:
					default  :
						OffsetX = myrect.fLeft;
						OffsetY = myrect.fTop;
				}
			}
			else  // zoom value set
			{
				OffsetX = myrect.fLeft-AreaHandle->ScrollX;
				OffsetY = myrect.fTop-AreaHandle->ScrollY; 
			}
			Rect destRect(OffsetX, OffsetY, (INT) SizeX, (INT) SizeY);  // GDI Rect expects X + Width, not X + X2

			Rect mainRect(myrect.fLeft, myrect.fTop, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
			Region updateregion(mainRect);
			updateregion.Exclude(destRect);

			graphics.SetClip(mainRect);  // this speeds up drawing drastically...
			graphics.DrawImage(AreaHandle->image, destRect);		

			SolidBrush solidBrush(Color(255, 255, 255, 255));
			solidBrush.SetColor(Color::Transparent);
			graphics.FillRegion(&solidBrush, &updateregion);
		}
		else
		{  // even if the image is empty, we should draw a white frame
			Rect mainRect(myrect.fLeft, myrect.fTop, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
			Region updateregion(mainRect);

			SolidBrush solidBrush(Color(255, 255, 255, 255));
			solidBrush.SetColor(Color::Transparent);
			graphics.FillRegion(&solidBrush, &updateregion);
		}
}
#else
void DrawImage(ImageAreaStruct *AreaHandle, PA_Rect myrect)
{
	long SizeX, SizeY;
	bool NewHShow, NewVShow;
	long yMaxScroll, Height, yNewSize, ViewSize; 

	if(AreaHandle->image != NULL)	
		{
		GetScaledImageSize(AreaHandle, SizeX, SizeY, myrect.fRight-myrect.fLeft, myrect.fBottom-myrect.fTop);
		// scrollbar handling, do nothing without scrollbars
		if (AreaHandle->HorScroll && AreaHandle->VertScroll)
			{
			// scrollbar - calculate if visible
			if ((AreaHandle->ZoomX <= 0) || (AreaHandle->ZoomY <= 0))  // automatic modus - hide them
				NewHShow = NewVShow = false;
			else
				{  // show if pict larger than space
				if ((!AreaHandle->HScrollShow) && (!AreaHandle->VScrollShow))
					NewHShow = NewVShow = false;
				else
					{	
					NewHShow = NewVShow = false;
					if (SizeX > (myrect.fRight - myrect.fLeft))
						if (AreaHandle->HorScroll && AreaHandle->HScrollShow)
							NewHShow = true;
					if (SizeY > (myrect.fBottom - myrect.fTop))
						if (AreaHandle->VertScroll && AreaHandle->VScrollShow)
							NewVShow = true;
					}
				}
			
			// Scrollbar reset
			if (AreaHandle->HorScroll && NewHShow)
				{	// horizontal	
				yNewSize = myrect.fRight - myrect.fLeft;
				if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
					yNewSize -= AreaScrollbarSize;					
				if(AreaHandle->image != NULL)
					Height = SizeX;
				else
					Height = 0;
				yMaxScroll = Height; 
				AreaHandle->ScrollX = min(AreaHandle->ScrollX, (Height-yNewSize));
				AreaHandle->ScrollX = min(AreaHandle->ScrollX, yMaxScroll); 
				if ((Height == 0) || (yMaxScroll == 0))
					ViewSize = 0;
				else
					ViewSize = (int) ((double) yNewSize / (double) Height * (double) yMaxScroll);
				
				SetControl32BitMaximum(AreaHandle->HorScroll,yMaxScroll-ViewSize);
				SetControl32BitValue(AreaHandle->HorScroll,AreaHandle->ScrollX);
				SetControlViewSize(AreaHandle->HorScroll,ViewSize);
				}
			if (AreaHandle->VertScroll && NewVShow)
				{	// vertical	
				yNewSize = myrect.fBottom - myrect.fTop;
				if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
					yNewSize -= AreaScrollbarSize;	
				if(AreaHandle->image != NULL)
					Height = SizeY;
				else
					Height = 0;
				yMaxScroll = Height; 
				AreaHandle->ScrollY = min(AreaHandle->ScrollY, (Height-yNewSize));
				AreaHandle->ScrollY = min(AreaHandle->ScrollY, yMaxScroll); 
				if ((Height == 0) || (yMaxScroll == 0))
					ViewSize = 0;
				else
					ViewSize = (int) ((double) yNewSize / (double) Height * (double) yMaxScroll);
				
				SetControl32BitMaximum(AreaHandle->VertScroll,yMaxScroll-ViewSize);
				SetControl32BitValue(AreaHandle->VertScroll,AreaHandle->ScrollY);
				SetControlViewSize(AreaHandle->VertScroll,ViewSize);
				}
			
			if (AreaHandle->HScrollActive != NewHShow)
				{
				AreaHandle->HScrollActive = NewHShow;
				if (AreaHandle->HScrollActive)
					ShowControl(AreaHandle->HorScroll); 
				else
					HideControl(AreaHandle->HorScroll);
				}
			if (AreaHandle->VScrollActive != NewVShow)
				{
				AreaHandle->VScrollActive = NewVShow;
				if (AreaHandle->VScrollActive)
					ShowControl(AreaHandle->VertScroll); 
				else
					HideControl(AreaHandle->VertScroll); 
				}
			
			// remove them if visible
			if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
				myrect.fBottom -= AreaScrollbarSize;
			if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
				myrect.fRight -= AreaScrollbarSize;
			}
				
		DrawImageSub_Mac(AreaHandle, myrect, SizeX, SizeY);
		}
}

void DrawImageSub_Mac(ImageAreaStruct *AreaHandle, PA_Rect myrect, long SizeX, long SizeY)
{
	long OffsetX, OffsetY;	
	
	if(AreaHandle->image != NULL)	
		{
	/*  already done before calling
		if (AreaHandle->HScrollActive && AreaHandle->HorScroll)
			myrect.fBottom -= AreaScrollbarSize;
		if (AreaHandle->VScrollActive && AreaHandle->VertScroll)
			myrect.fRight -= AreaScrollbarSize;
			*/

		if ((AreaHandle->ZoomX <= 0) || (AreaHandle->ZoomY <= 0))  // automatic modus
			{
			switch (AreaHandle->ZoomX)
			{	case GDI_ScaledToFit:
				OffsetX = myrect.fLeft;
				OffsetY = myrect.fTop;
				break;
			
				case GDI_ScaledToFitPropCentered:
					OffsetX = myrect.fLeft + abs((int) ((myrect.fRight-myrect.fLeft)-SizeX) / 2);
					OffsetY = myrect.fTop + abs((int) ((myrect.fBottom-myrect.fTop)-SizeY) / 2); 
					break;
					
				case GDI_ReduceToFitPropCentered:
					OffsetX = myrect.fLeft + abs((int) ((myrect.fRight-myrect.fLeft)-SizeX) / 2);
					OffsetY = myrect.fTop + abs((int) ((myrect.fBottom-myrect.fTop)-SizeY) / 2); 
					break;
					
				case GDI_ReduceToFitProp  :
					OffsetX = myrect.fLeft;
					OffsetY = myrect.fTop;
					break;
					
					// case GDI_ScaledToFitProp:
				default  :
					OffsetX = myrect.fLeft;
					OffsetY = myrect.fTop;
				}
		}
		else  // zoom value set
		{
			OffsetX = myrect.fLeft-AreaHandle->ScrollX;
			OffsetY = myrect.fTop-AreaHandle->ScrollY; 
		}

		SetRect((Rect *) &myrect, OffsetX, OffsetY, OffsetX+SizeX, OffsetY+SizeY);
		// clip Region on Mac should be already set
		DrawPicture(AreaHandle->image, (const Rect *) &myrect);

		}
	else
	{ // even if the image is empty, we should draw a white frame
		EraseRect((const Rect *) &myrect);
	}
}
#endif

void GetScaledImageSize(ImageAreaStruct *AreaHandle, long &ImageSizeX, long &ImageSizeY, long SpaceX, long SpaceY)
{	
	double RatioX, RatioY, SizeX = 0, SizeY = 0;
	long OrigImageSizeX, OrigImageSizeY;

if(AreaHandle->image != NULL)  // do we have a picture, works both on MacOS and Windows		
	{
#if VERSIONWIN
		OrigImageSizeX = AreaHandle->image->GetWidth();
		OrigImageSizeY = AreaHandle->image->GetHeight(); 
#else
		PicHandle thePicture = AreaHandle->image;
		OrigImageSizeX =(*thePicture)->picFrame.right - (*thePicture)->picFrame.left;
		OrigImageSizeY =(*thePicture)->picFrame.bottom - (*thePicture)->picFrame.top;
#endif
		if ((AreaHandle->ZoomX <= 0) || (AreaHandle->ZoomY <= 0))  // automatic modus
		{
			// both have to be same value
			if  (AreaHandle->ZoomX > 0) AreaHandle->ZoomX = AreaHandle->ZoomY;
			switch (AreaHandle->ZoomX)
			{	case GDI_ScaledToFit:
					SizeX = SpaceX; 
					SizeY = SpaceY; 
					break;

				case GDI_ScaledToFitPropCentered:
					RatioX = (double) (OrigImageSizeX) / (double) (SpaceX);
					RatioY = (double) (OrigImageSizeY) / (double) (SpaceY);
					if (RatioX < RatioY)  //  use width as max
					{	SizeX = (OrigImageSizeX) * (100/RatioY) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioY) / 100;
					}
					else
					{	SizeX = (OrigImageSizeX) * (100/RatioX) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioX) / 100;
					}
					break;

				case GDI_ReduceToFitPropCentered:
					RatioX = (double) (OrigImageSizeX) / (double) (SpaceX);
					RatioY = (double) (OrigImageSizeY) / (double) (SpaceY);
					if (RatioX < RatioY)  //  use width as max
					{	if (RatioY < 1) RatioY = 1;
						SizeX = (OrigImageSizeX) * (100/RatioY) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioY) / 100;
					}
					else
					{	if (RatioX <1) RatioX = 1;
						SizeX = (OrigImageSizeX) * (100/RatioX) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioX) / 100;
					}
					break;

				case GDI_ReduceToFitProp  :
					RatioX = (double) (OrigImageSizeX) / (double) (SpaceX);
					RatioY = (double) (OrigImageSizeY) / (double) (SpaceY);
					if (RatioX < RatioY)  //  use width as max
					{	if (RatioY <1) RatioY = 1;
						SizeX = (OrigImageSizeX) * (100/RatioY) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioY) / 100;
					}
					else
					{	if (RatioX <1) RatioX = 1;
						SizeX = (OrigImageSizeX) * (100/RatioX) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioX) / 100;
					}
					break;

				// case GDI_ScaledToFitProp:
				default  :
					RatioX = (double) (OrigImageSizeX) / (double) (SpaceX);
					RatioY = (double) (OrigImageSizeY) / (double) (SpaceY);
					if (RatioX < RatioY)  //  use width as max
					{	SizeX = (OrigImageSizeX) * (100/RatioY) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioY) / 100;
					}
					else
					{	SizeX = (OrigImageSizeX) * (100/RatioX) / 100;
						SizeY = (OrigImageSizeY) * (100/RatioX) / 100;
					}
				}
			}
			else  // zoom value set
			{
				SizeX = (OrigImageSizeX) * AreaHandle->ZoomX / 100;
				SizeY = (OrigImageSizeY) * AreaHandle->ZoomY / 100;
			}
	}
	ImageSizeX = (long) SizeX;
	ImageSizeY = (long) SizeY;
}

#if VERSIONWIN
void ConvertPA_RectToRect(PA_Rect source, RECT *dest)
   {
      if(dest)
      {
         dest->top = source.fTop;
         dest->left = source.fLeft;
         dest->bottom = source.fBottom;
         dest->right = source.fRight;
      }
   }

int GetEncoderClsid(const WCHAR* format, CLSID* pClsid)
{
   UINT  num = 0;          // number of image encoders
   UINT  size = 0;         // size of the image encoder array in bytes

   ImageCodecInfo* pImageCodecInfo = NULL;

   GetImageEncodersSize(&num, &size);
   if(size == 0)
      return -1;  // Failure

   pImageCodecInfo = (ImageCodecInfo*)(malloc(size));
   if(pImageCodecInfo == NULL)
      return -1;  // Failure

   GetImageEncoders(num, size, pImageCodecInfo);

   for(UINT j = 0; j < num; ++j)
   {
      if( wcscmp(pImageCodecInfo[j].MimeType, format) == 0 )
      {
         *pClsid = pImageCodecInfo[j].Clsid;
         free(pImageCodecInfo);
         return j;  // Success
      }    
   }

   free(pImageCodecInfo);
   return -1;  // Failure
}

int GetEncoderClsid(char * format, CLSID &pngClsid)
{
		pngClsid = ImageFormatUndefined;
		// GetEncoderClsid(L"image/jpg", &pngClsid);   // for unknown formats
		if (!strcmp(format, "jpg"))
			return GetEncoderClsid(L"image/jpeg", &pngClsid);
		if (!strcmp(format, "tif"))
			return GetEncoderClsid(L"image/tiff", &pngClsid);
		if (!strcmp(format, "bmp"))
			return GetEncoderClsid(L"image/bmp", &pngClsid);
		if (!strcmp(format, "png"))
			return GetEncoderClsid(L"image/png", &pngClsid);
		if (!strcmp(format, "gif"))
			return GetEncoderClsid(L"image/gif", &pngClsid);

		return -1;
}
#endif

/*
 INTERNAL USE - MAC ONLY
 We are able to open a fullpath > 255, and unicode.
 */
#if VERSIONMAC
// NewPath		can be NULL		Encoding MacRoman
// newPathUTF8  can be NULL		Encoding UTF8
OSErr build_4DPath_2_PosixPath (char* Pfad , char *NewPath , UInt8 *newPathUTF8 , CFIndex buffSize  )
{
	OSErr		err			= noErr;
	CFStringRef filePath	= NULL;
	CFURLRef	url			= NULL;
	CFStringRef altpath		= NULL ;
	TextEncoding			myTextEncoding = kCFStringEncodingMacRoman;  
	
	filePath = CFStringCreateWithBytes ( kCFAllocatorDefault,
										 (const UInt8*) Pfad,
										 (CFIndex) strlen(Pfad),
										 myTextEncoding, /* CFStringEncoding encoding,*/
										 false);
	
	if ( filePath != NULL )
		{
		url = CFURLCreateWithFileSystemPath(NULL, filePath, kCFURLHFSPathStyle, false);
		if ( url != NULL )
			{
			altpath = CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle);
			if ( altpath != NULL )
				{
				if ( newPathUTF8 != NULL)
					CFStringGetCString (altpath, (char*)newPathUTF8, buffSize, kCFStringEncodingUTF8 );
				
				if ( NewPath != NULL)
					CFStringGetCString (altpath, NewPath, buffSize, kCFStringEncodingMacRoman );				
				}
			}
		}
	
	if ( filePath != NULL )
		CFRelease(filePath);
	else
		err = kErr_MemError;
	
	if ( url != NULL )
		CFRelease(url);
	else
		err = kErr_MemError;
	
	if ( altpath != NULL )
		CFRelease(altpath);
	else
		err = kErr_MemError;
	
	return ( err );
	
}


OSType GetEncoderType(char * format)
{
		// GetEncoderClsid(L"image/jpg", &pngClsid);   // for unknown formats
		if (!strcmp(format, "jpg"))
			return kQTFileTypeJPEG;
		if (!strcmp(format, "tif"))
			return kQTFileTypeTIFF;
		if (!strcmp(format, "bmp"))
			return kQTFileTypeBMP;
		if (!strcmp(format, "png"))
			return kQTFileTypePNG;
		if (!strcmp(format, "gif"))
			return kQTFileTypeGIF;
		if (!strcmp(format, "Pict"))
			return kQTFileTypePicture;		
			
		return 0;
}

long GetCompression(long compression)
{
	// convert Windows Compression 0..100 in Mac Compression 0..0x3FF (1023)
	double result;
	
	result = 0x400 / (100 / compression);
	if (result >= 0x400) result = 0x3FF;
	return (long) result;	
}

//  this routine is useable on Windows if quickTime is linked. we do not need it in this plugin on Windows
long build_FSSpecFrompath ( char* thePath, FSSpec *myFSSpec)
{
	long		err					= noErr;
	long		pathLen				= 0;
#if VERSIONMAC
	Boolean		isDirectory		= false;
	UInt8		*NewPath_utf8   = NULL;
	FSRef		FSRef;
#endif
	
	
#if VERSIONWIN
	err = NativePathNameToFSSpec ( thePath , myFSSpec , kErrorIfFileNotFound );
#else			
	pathLen=strlen(thePath) * 3;
	NewPath_utf8 = (UInt8*) malloc ( pathLen );  // to have enough space for diacritical chars
	if ( NewPath_utf8 != NULL ) 
		err = build_4DPath_2_PosixPath (thePath , 0L , NewPath_utf8 , pathLen );
	else
		err = kErr_MemError;
	
	if (( NewPath_utf8 != NULL ) && ( err == noErr ))
		err = FSPathMakeRef ( NewPath_utf8 , &FSRef, &isDirectory);
	
	if ( err == noErr )
		err = FSGetCatalogInfo ( &FSRef , kFSCatInfoNone ,
								 NULL, NULL,  myFSSpec,  NULL );
	
	if ( NewPath_utf8 != NULL )
		free ( NewPath_utf8);
#endif
	
	return err;
	
}

// this routine is similar to build_FSSpecFrompath, but more complex
// it allows to build a FSSpec for a not existing file, which FSPathMakeRef does not support
// we need this to save a new file

long build_FSSpecFrompathNewfile ( char* thePath, FSSpec *myFSSpec)
{
	long		err					= noErr;
	FSRef		FSRef;
    CFStringRef cfstring = NULL;
	CFStringRef nameString = NULL;
	CFURLRef outurl = NULL;
	CFURLRef parentURL = NULL;
	FSCatalogInfo info;

	
	cfstring = CFStringCreateWithCString(NULL, thePath, kCFStringEncodingMacRoman);
	if (cfstring)
		outurl = CFURLCreateWithFileSystemPath(NULL, cfstring, kCFURLHFSPathStyle,false);
	else
		err = memFullErr;

	// Get a CFURL for the parent
	if (outurl)
         if (err == noErr) {
            parentURL = CFURLCreateCopyDeletingLastPathComponent (NULL, outurl);
            if (parentURL == NULL)  err = memFullErr; 
        }
			 else err = memFullErr;
				 
			 
	if (err == noErr) {
		Boolean converted = CFURLGetFSRef (parentURL, &FSRef);
		if (!converted) { err = fnfErr; } 
	}
	
	// Get the node ID of the parent directory
	if (err == noErr) {
		err = FSGetCatalogInfo(&FSRef, kFSCatInfoNodeFlags|kFSCatInfoNodeID, &info, NULL, myFSSpec, NULL);
	}
	
	// Get a CFString for the file name
	if (err == noErr) {
		nameString = CFURLCopyLastPathComponent (outurl);
		if (nameString == NULL) { err = memFullErr; }
	}
	
	// Copy the string into the FSSpec
	if (err == noErr) {	
		Boolean converted = CFStringGetPascalString (nameString, myFSSpec->name, sizeof (myFSSpec->name), 
													 CFStringGetSystemEncoding ());
		if (!converted) { err = fnfErr; }
		
	}
    
	// Set the node ID in the FSSpec
	if (err == noErr) {
		myFSSpec->parID = info.nodeID;
	}		
			
	if (outurl) free((void*)outurl);
	if (parentURL) free((void*)parentURL);
    if (cfstring) free((void*)cfstring);
    if (nameString) free((void*)nameString);

	return err;
}
#endif

long CheckAssignedField( PA_PluginParameters params,  bool structurecall )
{
	char areaname[255], name[255];
	ImageAreaStruct *AreaHandle;
	PA_FieldKind   kind;
	short         len;
	char         isIndexed;
	long         attrs;
	short		i;
	
	AreaHandle = (ImageAreaStruct *) PA_GetLongParameter( params, 1 );
	PA_GetAreaName(params, areaname);	
	strlcat(areaname, "_", 255);
	short tablenum;
	if (structurecall)
		{
		PA_PluginProperties currentPluginProps;	 
		PA_GetPluginProperties(params, &currentPluginProps);
		tablenum = currentPluginProps.fTable;
		}
	else
		tablenum = AreaHandle->TableNumber;
	short count = PA_CountFields (tablenum);
	for(i = 1; i <= count; i++)
		{
		PA_GetFieldName(tablenum, i, name);
		// add "_"
		
		if (strcmp(name, areaname) == 0)
			{
			PA_GetFieldProperties(tablenum, i, &kind, &len, &isIndexed, &attrs);
			if(PA_GetLastError())
				return -1;
			switch (kind)
				{
					case eFK_PictureField:
						return eFK_PictureField; break;
					case eFK_BlobField:
						return eFK_BlobField; break;
					case eFK_TextField:
						return eFK_TextField; break;
					default:
						;   // nothing, continue loop
				}
			}
		}
	
	
	return 0;
}


void LoadAssignedField( PA_PluginParameters params)
{
	char areaname[255], name[255];
	PA_FieldKind   kind;
	char         isIndexed;
	long         attrs;
	short i, fieldlen=0;
	
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
		{	
		AreaHandle->Dirty = false;
		PA_GetAreaName(params, areaname);	
		strlcat(areaname, "_", 255);
		short tablenum;
		tablenum = AreaHandle->TableNumber;
		short count = PA_CountFields (tablenum);
		for(i = 1; i <= count; i++)
			{
			PA_GetFieldName(tablenum, i, name);
			// add "_"
			
			if (strcmp(name, areaname) == 0)
				{
				PA_GetFieldProperties(tablenum, i, &kind, &fieldlen, &isIndexed, &attrs);
				if(PA_GetLastError())
					return;
				long len = 0;
				switch (kind)
					{
					case eFK_PictureField:
						PA_PictureInfo   theInfo;
						Load4DPictureInHandle(AreaHandle, PA_GetPictureHandleField( tablenum, i, &theInfo ) );
						return;
						break;
					case eFK_BlobField:
						LoadBlobInHandle(AreaHandle, PA_GetBlobHandleField( tablenum, i ) );
						return;
						 break;
					case eFK_TextField:
						len = PA_GetTextField(tablenum, i, 0 );
						if ( PA_GetLastError() == eER_NoErr )
							{
							char * textpara;
							textpara = (char *) malloc( len+1 );
							if ( textpara )
								{
								PA_GetTextField(tablenum, i, textpara );
								textpara[len] = 0;
								LoadPictureFile(AreaHandle, textpara);
								free( textpara );
								}
							}
						return;
						break;
					default:
						;   // nothing, continue loop
					}
				}
			}
		}
}


void SaveAssignedField( PA_PluginParameters params)
{
	char areaname[255], name[255];
	PA_FieldKind   kind;
	char         isIndexed;
	long         attrs;
	short i, fieldlen=0;
	
	ImageAreaStruct *AreaHandle;
	
	AreaHandle = (ImageAreaStruct *) PA_GetAreaReference( params );
	if ((AreaHandle != NULL) && (AreaHandle->TAG == kAreaTag))
	  if (AreaHandle->Dirty)
		{
		PA_Event* event;
		event = ( (PA_Event**) params->fParameters )[ 0 ];
		if ( event->fWhen == 0 )
		{  // form accepted, 1 for Cancel
			AreaHandle->Dirty = false;
			PA_GetAreaName(params, areaname);	
			strlcat(areaname, "_", 255);
			short tablenum;
			tablenum = AreaHandle->TableNumber;
			short count = PA_CountFields (tablenum);
			for(i = 1; i <= count; i++)
				{
				PA_GetFieldName(tablenum, i, name);
				// add "_"
				
				if (strcmp(name, areaname) == 0)
					{
					PA_GetFieldProperties(tablenum, i, &kind, &fieldlen, &isIndexed, &attrs);
					if(PA_GetLastError())
						return;
					switch (kind)
						{
						case eFK_PictureField:
							Save4DPicture(AreaHandle, NULL, 0, tablenum, i, 0);
							return;
							break;
						case eFK_BlobField:
							SavePictureBlob(AreaHandle, NULL, "JPEG", 50, 0, tablenum, i);
							return;
							 break;
						case eFK_TextField:
							// nothing !!!
							return;
							break;
						default:
							;   // nothing, continue loop
						}
					}
				}
			}
		}
}


short PA_GetModifiers( PA_PluginParameters params )
{
	PA_Event* event;
	
	event = ( (PA_Event**) params->fParameters )[ 0 ];
	if ( event->fWhat == eAE_KeyDown || event->fWhat == eAE_AutoKey || event->fWhat == eAE_MouseDown )
		return event->fModifiers ;
	else
		return 0;
}
